# -------------------------------------------------------------
# One-probe Configuration
# -------------------------------------------------------------

# -------------------------------------------------------------
# Left Electrode
# -------------------------------------------------------------

# Set up lattice
vector_a = [2.93319114416, 0.0, 0.0]*Angstrom
vector_b = [0.0, 2.93319114416, 0.0]*Angstrom
vector_c = [0.0, 0.0, 8.2963173942]*Angstrom
left_electrode_lattice = UnitCell(vector_a, vector_b, vector_c)

# Define elements
left_electrode_elements = [Silver, Silver, Silver, Silver]

# Define coordinates
left_electrode_coordinates = [[ 1.466595572079,  1.466595572079,  1.037039674275],
                              [ 0.            ,  0.            ,  3.111119022825],
                              [ 1.466595572079,  1.466595572079,  5.185198371375],
                              [ 0.            ,  0.            ,  7.259277719925]]*Angstrom

# Set up configuration
left_electrode = BulkConfiguration(
    bravais_lattice=left_electrode_lattice,
    elements=left_electrode_elements,
    cartesian_coordinates=left_electrode_coordinates
    )

# -------------------------------------------------------------
# Central Region
# -------------------------------------------------------------

# Set up lattice
vector_a = [2.93319114416, 0.0, 0.0]*Angstrom
vector_b = [0.0, 2.93319114416, 0.0]*Angstrom
vector_c = [0.0, 0.0, 30.7407934855]*Angstrom
central_region_lattice = UnitCell(vector_a, vector_b, vector_c)

# Define elements
central_region_elements = [Silver, Silver, Silver, Silver, Silver, Silver, Silver, Silver,
                           Silver, Silver]

# Define coordinates
central_region_coordinates = [[  1.466595572079,   1.466595572079,   1.037039674275],
                              [  0.            ,   0.            ,   3.111119022825],
                              [  1.466595572079,   1.466595572079,   5.185198371375],
                              [  0.            ,   0.            ,   7.259277719925],
                              [  1.466595572079,   1.466595572079,   9.333357068475],
                              [  0.            ,   0.            ,  11.407436417025],
                              [  1.466595572079,   1.466595572079,  13.481515765575],
                              [  0.            ,   0.            ,  15.555595114125],
                              [  1.466595572079,   1.466595572079,  17.629674462675],
                              [  0.            ,   0.            ,  19.703753811225]]*Angstrom

# Define ghost atoms
central_region_ghost_atoms = [9]

# Set up configuration
central_region = BulkConfiguration(
    bravais_lattice=central_region_lattice,
    elements=central_region_elements,
    cartesian_coordinates=central_region_coordinates,
    ghost_atoms=central_region_ghost_atoms
    )

surface_configuration = SurfaceConfiguration(
    central_region,
    left_electrode,
    )

# -------------------------------------------------------------
# Calculator
# -------------------------------------------------------------
#----------------------------------------
# Basis Set
#----------------------------------------
basis_set = [
    LDABasis.Silver_SingleZetaPolarized,
    ]

#----------------------------------------
# Numerical Accuracy Settings
#----------------------------------------
left_electrode_k_point_sampling = MonkhorstPackGrid(
    na=9,
    nb=9,
    nc=100,
    )
left_electrode_numerical_accuracy_parameters = NumericalAccuracyParameters(
    k_point_sampling=left_electrode_k_point_sampling,
    )

device_k_point_sampling = MonkhorstPackGrid(
    na=9,
    nb=9,
    nc=100,
    )
device_numerical_accuracy_parameters = NumericalAccuracyParameters(
    k_point_sampling=device_k_point_sampling,
    )

#----------------------------------------
# Poisson Solver Settings
#----------------------------------------
left_electrode_poisson_solver = FastFourier2DSolver(
    boundary_conditions=[[PeriodicBoundaryCondition(),PeriodicBoundaryCondition()],
                         [PeriodicBoundaryCondition(),PeriodicBoundaryCondition()],
                         [PeriodicBoundaryCondition(),PeriodicBoundaryCondition()]]
    )

device_poisson_solver = FastFourier2DSolver(
    boundary_conditions=[[PeriodicBoundaryCondition(),PeriodicBoundaryCondition()],
                         [PeriodicBoundaryCondition(),PeriodicBoundaryCondition()],
                         [DirichletBoundaryCondition(),NeumannBoundaryCondition()]]
    )

#----------------------------------------
# Electrode Calculators
#----------------------------------------
left_electrode_calculator = LCAOCalculator(
    basis_set=basis_set,
    numerical_accuracy_parameters=left_electrode_numerical_accuracy_parameters,
    poisson_solver=left_electrode_poisson_solver,
    )

right_electrode_calculator = LCAOCalculator(
    basis_set=basis_set,
    )

#----------------------------------------
# Device Calculator
#----------------------------------------
calculator = DeviceLCAOCalculator(
    basis_set=basis_set,
    numerical_accuracy_parameters=device_numerical_accuracy_parameters,
    poisson_solver=device_poisson_solver,
    electrode_calculators=
        [left_electrode_calculator, right_electrode_calculator],
    )

surface_configuration.setCalculator(calculator)
nlprint(surface_configuration)
surface_configuration.update()
nlsave('silver_gfs.nc', surface_configuration)

# -------------------------------------------------------------
# Chemical Potential
# -------------------------------------------------------------
chemical_potential = ChemicalPotential(surface_configuration)
nlsave('silver_gfs.nc', chemical_potential)
nlprint(chemical_potential)

# -------------------------------------------------------------
# Electrostatic Difference Potential
# -------------------------------------------------------------
electrostatic_difference_potential = ElectrostaticDifferencePotential(surface_configuration)
nlsave('silver_gfs.nc', electrostatic_difference_potential)